﻿using System;
using System.Collections.Generic;               // !
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;                              // !
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApplication6
{

    public partial class Form1 : Form
    {
        // Beispieldaten ----------------------------------------------------------------------------------------------- 
        
        private static CKunde[] Kunden;                // Liste der Kunden mit ihren Bestellungen 
        private static CProdukt[] Produkte;            // Liste der Produkte

        static void ReadData()        
        {
            Kunden = new CKunde[] {
                new CKunde {Name = "Walter", Ort = "Altenburg", Land = ELänder.Germany, Bestellungen = new CBestellung[] {
                    new CBestellung {ProduktNr = 2, Anzahl = 4, Monat = "März", versendet = false}}},
                new CKunde {Name = "Thomas", Ort = "Berlin", Land = ELänder.Germany, Bestellungen =  new CBestellung[] {
                    new CBestellung {ProduktNr = 1, Anzahl = 11, Monat = "Juni", versendet = false},
                    new CBestellung {ProduktNr = 3, Anzahl = 19, Monat = "November", versendet = true}}},
                new CKunde {Name = "David", Ort = "Washington", Land = ELänder.USA, Bestellungen =  new CBestellung[] {
                    new CBestellung {ProduktNr = 5, Anzahl = 17, Monat = "November", versendet = true}}},
                new CKunde {Name = "Charlie", Ort = "New York", Land = ELänder.USA, Bestellungen =  new CBestellung[] {
                    new CBestellung {ProduktNr = 6, Anzahl = 12, Monat = "Juni", versendet = false}}},
                new CKunde {Name = "Alice", Ort = "London", Land = ELänder.GB, Bestellungen =  new CBestellung[] {
                    new CBestellung {ProduktNr = 4, Anzahl = 3, Monat = "Februar", versendet = true},
                    new CBestellung {ProduktNr = 2, Anzahl = 1, Monat = "Februar", versendet = false},
                    new CBestellung {ProduktNr = 3,Anzahl = 19, Monat = "Juni", versendet = true}}}};

            Produkte = new CProdukt[] {
                new CProdukt {ProduktNr = 1, Name = "Marmelade", Preis = 5 },
                new CProdukt {ProduktNr = 2, Name = "Quark", Preis = 10 },
                new CProdukt {ProduktNr = 3, Name = "Mohrrüben", Preis = 15 },
                new CProdukt {ProduktNr = 4, Name = "Käse", Preis = 20 },
                new CProdukt {ProduktNr = 5, Name = "Honig", Preis = 25 },
                new CProdukt {ProduktNr = 6, Name = "Mehl", Preis = 30 }};
        }

        public Form1()
        {
            InitializeComponent();
            ReadData();
        }

        // --------------------- LINQ-Abfragen ------------------------------------------------------------------------------------

        // Where 1  (Query Expression Syntax)
        private void button1_Click(object sender, EventArgs e)
        {
            var expr =
                from k in Kunden
                where k.Land == ELänder.Germany
                select new { k.Name, k.Ort };

            foreach (var kd in expr)
                      listBox1.Items.Add(kd); 
        }

        // Where 2 (Extension Method Syntax mit Indexfilter)
        private void button2_Click(object sender, EventArgs e)
        {
            var expr =
                Kunden
                .Where((k, index) => ((index >= 2) && (index < 4)))
                .Select(k => k.Name);

            foreach (var kd in expr)
                listBox1.Items.Add(kd); 
        }

        // Select 1
        private void button3_Click(object sender, EventArgs e)
        {
            var allProdukte = from p in Produkte select p.Name;  // Query Expression Syntax
            // var allProdukte = Produkte.Select(p => p.Name);   // Extension Method Syntax
            foreach (var p in allProdukte)
                listBox1.Items.Add(p);    
        }

        // Select 2  (Extension Method Syntax)
        private void button4_Click(object sender, EventArgs e)
        {
           var expr = Kunden.Select(k => new { k.Name, k.Ort }); 
            
           foreach (var k in expr)
               listBox1.Items.Add(k);    
        }

        // SelectMany 1  (Extension Method Syntax)
        private void button5_Click(object sender, EventArgs e)
        {
            var bestellungen = Kunden                           
                .Where(k => k.Land == ELänder.Germany)
                .SelectMany(k => k.Bestellungen);
            foreach (var b in bestellungen)
                listBox1.Items.Add(b);
        }

        // SelectMany 2 (dasselbe in Query Expression Syntax)
        private void button6_Click(object sender, EventArgs e)
        {
            var bestellungen =
                from k in Kunden
                where k.Land == ELänder.Germany
                from b in k.Bestellungen
                select b;
            foreach (var b in bestellungen)
                listBox1.Items.Add(b);
        }

        // OrderBy 1 (Extension Method Syntax)
        private void button7_Click(object sender, EventArgs e)
        {
            var prod = Produkte
                .Where(p => p.Preis <= 20)
                .OrderByDescending(p => p.Preis)
                .Select(p => new { p.Name, p.Preis });
            foreach (var p in prod)
                listBox1.Items.Add(p);
        }

        // OrderBy 2  (dasselbe in Query Expression Syntax)
        private void button8_Click(object sender, EventArgs e)
        {
            var prod =
                from p in Produkte
                where p.Preis <= 20
                orderby p.Preis descending
                select new { p.Name, p.Preis };
            foreach (var p in prod)
                listBox1.Items.Add(p);
        }

        // ThenBy 1   (Extension Method Syntax)
        private void button9_Click(object sender, EventArgs e)
        {
            var knd = Kunden
                      .OrderBy(k => k.Land)
                      .ThenBy(k => k.Name)
                      .Select(k => new { k.Land, k.Name});
            foreach (var k in knd)
                listBox1.Items.Add(k);
        }

        // ThenBy 2  (dasselbe in Query Expression Syntax)
        private void button10_Click(object sender, EventArgs e)
        {
            var knd = from k in Kunden
                      orderby k.Land, k.Name
                      select new { k.Land, k.Name };
            foreach (var ku in knd)
                listBox1.Items.Add(ku);
        }

        // Reverse  (Extension Method Syntax)
        private void button11_Click(object sender, EventArgs e)
        {
            var knd = Kunden.OrderBy(k => k.Land).ThenBy(k => k.Name).Select(k => new { k.Land, k.Name })
                      .Reverse();
            foreach (var k in knd)
                listBox1.Items.Add(k);
        }

        // GroupBy 1  (Extension Method Syntax)
        private void button12_Click(object sender, EventArgs e)
        {            
            var knd = Kunden.GroupBy(k => k.Land);
            foreach (IGrouping<ELänder, CKunde> kdGroup in knd)  // anstatt IGrouping<ELänder, CKunde> auch nur var möglich
            {
                listBox1.Items.Add(kdGroup.Key);
                foreach (var kd in kdGroup)
                    listBox1.Items.Add("  " + kd);
            }           
        }

        // GroupBy 2    (dasselbe in Query Expression Syntax)
        private void button13_Click(object sender, EventArgs e)
        {
            var knd = from k in Kunden
                      group k by k.Land;

            foreach (var kdGroup in knd)                        // anstatt var: IGrouping<ELänder, CKunde>
            {
                listBox1.Items.Add(kdGroup.Key);
                foreach (var kd in kdGroup)
                    listBox1.Items.Add("  " + kd);
            }           
        }

        // GroupBy 3 (Extension Method Syntax)
        private void button14_Click(object sender, EventArgs e)
        {
            var prodGroups = Produkte
                            .GroupBy(p => p.Name[0], p => p.Name);
            foreach (var pGroup in prodGroups)                 // anstatt var: IGrouping<char, string> 
            {
                listBox1.Items.Add(pGroup.Key);
                foreach (var p in pGroup)
                    listBox1.Items.Add("  " + p);
            }
        }

        // GroupBy 4  (dasselbe in Query Expression Syntax)
        private void button15_Click(object sender, EventArgs e)
        {
            var prodGroups = from p in Produkte
                             group p by p.Name[0] into g
                             select new { firstLetter = g.Key, prods = g };
            foreach (var pGroup in prodGroups)
            {
                listBox1.Items.Add(pGroup.firstLetter);
                foreach (var p in pGroup.prods)
                    listBox1.Items.Add("  " + p.Name);
            }
        }

        // GroupBy 5  (Extension Method Syntax)
        private void button16_Click(object sender, EventArgs e)
        {
            var knd = Kunden
                      .GroupBy(k => k.Land, k => k.Name);
            foreach (var kdGroup in knd)
            {
                listBox1.Items.Add(kdGroup.Key);
                foreach (var kd in kdGroup)
                    listBox1.Items.Add("  " + kd);
            }    
        }

        // Join 1  (Extension Method Syntax)
        private void button17_Click(object sender, EventArgs e)
        {
            var bestprod = Kunden.SelectMany(k => k.Bestellungen)
                             .Join(Produkte, b => b.ProduktNr, p => p.ProduktNr,
                             (b, p) => new { b.Monat, p.ProduktNr, p.Name, p.Preis, b.versendet });
            foreach (var bp in bestprod)
            {
                listBox1.Items.Add(bp);
            }
 
        }

        // Join 2  (dasselbe in Query Expression Syntax)
        private void button18_Click(object sender, EventArgs e)
        {
            var bestprod = from k in Kunden
                           from b in k.Bestellungen
                           join p in Produkte on b.ProduktNr equals p.ProduktNr
                           select new { b.Monat, p.ProduktNr, p.Name, p.Preis, b.versendet };
            foreach (var bp in bestprod)
            {
                listBox1.Items.Add(bp);
            }
        }

        // Count 1  (Extension Method Syntax)
        private void button19_Click(object sender, EventArgs e)
        {
            var kdn = Kunden.Select(k => new { k.Name, k.Ort, AnzahlBest = k.Bestellungen.Count() });
            foreach (var ku in kdn)
                listBox1.Items.Add(ku);
        }

        // Count 2 (dasselbe in Query Expression Syntax)
        private void button20_Click(object sender, EventArgs e)
        {
            var kdn = from k in Kunden
                      select new { k.Name, k.Ort, AnzahlBest = k.Bestellungen.Count() };
            foreach (var ku in kdn)
                listBox1.Items.Add(ku);
        }

        // Sum 1  (Extension Method Syntax)
        private void button21_Click(object sender, EventArgs e)
        {
            var total = Produkte.Sum(p => p.Preis);
            listBox1.Items.Add(total);
        }

        // Sum 2 (dasselbe in Query Expression Syntax)
        private void button22_Click(object sender, EventArgs e)
        {
            var total = (from p in Produkte select p.Preis).Sum();
            listBox1.Items.Add(total);
        }

        // Komplexbeispiel
        private void button23_Click(object sender, EventArgs e)
        {
            var expr = from k in Kunden
                       join b in
                           (
                               from k in Kunden
                               from b in k.Bestellungen
                               join p in Produkte
                               on b.ProduktNr equals p.ProduktNr
                               select new { k.Name, BestellBetrag = b.Anzahl * p.Preis }
                               ) on k.Name equals b.Name
                       into KundenMitBest
                       select new { k.Name, TotalBetrag = KundenMitBest.Sum(b => b.BestellBetrag) };
            foreach (var k in expr)
                listBox1.Items.Add(k);
        }

        // Verzögerte Auswertung
        private void button24_Click(object sender, EventArgs e)
        {
            var MProds = Produkte.Where(p => p.Name[0] == 'M').Select(p => p.Name);
            //var MProds = (from p in Produkte where p.Name[0] == 'M' select p.Name);
            foreach (var prod in MProds)
                  listBox1.Items.Add(prod);
            listBox1.Items.Add("-----------");
            Produkte[0].Name = "Milch";         // Source-Folge wird geändert 
            foreach (var prod in MProds)
                listBox1.Items.Add(prod);
        }


        // Unverzögerte Auswertung
        private void button25_Click(object sender, EventArgs e)
        {
            var MProds = (Produkte.Where(p => p.Name[0] == 'M').Select(p => p.Name)).ToArray();
            //var MProds = (from p in Produkte where p.Name[0] == 'M'  select p.Name).ToArray();
           foreach (var s in MProds)
               listBox1.Items.Add(s);
           listBox1.Items.Add("-----------");
            Produkte[0].Name = "Milch";           // Source-Folge wird geändert 
            foreach (var s in MProds)
                listBox1.Items.Add(s);
        }

        // Inhalt löschen
        private void button26_Click(object sender, EventArgs e)
        {
            listBox1.Items.Clear();
        }

        // Beenden
        private void button27_Click(object sender, EventArgs e)
        {
            this.Close();
        }

       
    }

    // --------------------------- Datenstrukturen -----------------------------------------------------------------------

    public enum ELänder
    {
        USA,
        GB,
        Germany
    }

    public class CKunde
    {
        public string Name;
        public string Ort;
        public ELänder Land;
        public CBestellung[] Bestellungen;

        public override string ToString()
        {
            return String.Format("{0} - {1} - {2}", this.Name, this.Ort, this.Land);
        }

        public static CKunde Empty      // liefert neues leeres Kundenobjekt
        {
            get
            {
                CKunde kd = new CKunde();
                kd.Name = String.Empty;
                kd.Land = ELänder.Germany;
                kd.Ort = String.Empty;
                kd.Bestellungen = (new List<CBestellung>(Enumerable.Empty<CBestellung>())).ToArray();

                return (kd);
            }
        }
    }

    public class CBestellung
    {
        public int ProduktNr;
        public int Anzahl;
        public string Monat;
        public bool versendet;
 
        public override string ToString()
        {
            return String.Format("ProdNr: {0} , Anzahl: {1} , Monat: {2} , Versand: {3}", this.ProduktNr, this.Anzahl, this.Monat, this.versendet);
        }

        public override bool Equals(object obj)
        {
            if (!(obj is CBestellung))
                return false;
            else
            {
                CBestellung b = (CBestellung)obj;
                return (b.ProduktNr == this.ProduktNr && b.Monat == this.Monat && b.Anzahl == this.Anzahl && b.versendet == this.versendet);
            }
        }

        public override int GetHashCode()
        {
            return String.Format("{0}|{1}|{2}|{3}", this.ProduktNr, this.Monat, this.Anzahl, this.versendet).GetHashCode();
        }
    }

    public class CProdukt
    {
        public int ProduktNr;
        public string Name;
        public decimal Preis;
        

        public override string ToString()
        {
            return String.Format("{0} - {1}", this.ProduktNr, this.Preis);
        }
    }

    
}
