using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace DataTable_BinaryFile
{
    using System.IO;

    public partial class Form1 : Form
    {
        private DataTable dt = null;

        private DataTable getDataTable()
        {
            DataTable dt = new DataTable("Belege");
            DataColumn col0 = dt.Columns.Add("Nr", typeof(System.Int32));
            col0.AutoIncrement = true;
            col0.AutoIncrementStep = 1;
            DataColumn col1 = dt.Columns.Add("EingangsDatum", typeof(System.DateTime));
            col1.AllowDBNull = false;
            col1.DefaultValue = DateTime.Now;
            DataColumn col2 = dt.Columns.Add("KuNr", typeof(System.Int32));
            col2.AllowDBNull = false;
            DataColumn col3 = dt.Columns.Add("GesamtNetto", typeof(System.Decimal));
            col3.DefaultValue = 0;
            DataColumn col4 = dt.Columns.Add("Bemerkung", typeof(System.String));
            col4.DefaultValue = "";
            col4.MaxLength = 50;
            return dt;
        }

        public Form1()
        {
            InitializeComponent();

            dt = getDataTable();
            dataGridView1.DataSource = dt;      // Datengitter an DataTable anbinden
            formatDataGridView(dataGridView1);  // ... und formatieren
        }

        // in Datei speichern:
        private void button2_Click(object sender, EventArgs e)
        {
            FileStream wStream = new FileStream("Belege.dat", FileMode.OpenOrCreate, FileAccess.Write);
	        BinaryWriter bWriter = new BinaryWriter(wStream);
            bWriter.Write(dt.Rows.Count);             // am Dateianfang steht die Anzahl der Datenstze!
      	    foreach (DataRow rw in dt.Rows)
      	    {	
                bWriter.Write((int)(rw["Nr"]));
		        bWriter.Write(rw["EingangsDatum"].ToString());
		        bWriter.Write((int)(rw["KuNr"]));
		        bWriter.Write((decimal)(rw["GesamtNetto"]));
		        bWriter.Write(rw["Bemerkung"].ToString());
	        }
	        bWriter.Flush();   			// Puffer => Datei
	        bWriter.Close();
	        wStream.Close();
        }

        // von Datei laden:
        private void button1_Click(object sender, EventArgs e)
        {
            FileStream rStream = new FileStream("Belege.dat", FileMode.OpenOrCreate, FileAccess.Read);
	        BinaryReader bReader = new BinaryReader(rStream);
	        int max  = bReader.ReadInt32();   
            if (rStream.Length > 0)                 // kein Lesen bei leerer bzw. neu angelegter Datei
	        {    
           	      for (int i=1; i<= max; i++)    
		          {	
                      DataRow rw = dt.NewRow();   // neue DataRow mit Struktur der DataTable erzeugen
                	  rw["Nr"] = bReader.ReadInt32();
                	  rw["EingangsDatum"] = Convert.ToDateTime(bReader.ReadString());
                	  rw["KuNr"] = bReader.ReadInt32();
                	  rw["GesamtNetto"] = bReader.ReadDecimal();
			          rw["Bemerkung"] =	bReader.ReadString();
                	  dt.Rows.Add(rw);   
 		        }
	       }
          	bReader.Close();
          	rStream.Close();
        }

        // Anzeige lschen:
        private void button3_Click(object sender, EventArgs e)
        {
            dt.Clear();  
        }

        // Spalten des DataGridView formatieren:
        private void formatDataGridView(DataGridView dgv)
        {
            // Datum formatieren:
            dgv.Columns.Remove("EingangsDatum");
            DataGridViewTextBoxColumn tbc1 = new DataGridViewTextBoxColumn();
            tbc1.DataPropertyName = "EingangsDatum";
            tbc1.HeaderText = "EingangsDatum";
            tbc1.Width = 90;
            tbc1.DefaultCellStyle.Format = "d";
            tbc1.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleCenter;
            tbc1.DisplayIndex = 1;
            dgv.Columns.Add(tbc1);

            // Whrung formatieren:
            dgv.Columns.Remove("GesamtNetto");
            DataGridViewTextBoxColumn tbc2 = new DataGridViewTextBoxColumn();
            tbc2.DataPropertyName = "GesamtNetto";
            tbc2.HeaderText = "GesamtNetto";
            tbc2.Width = 80;
            tbc2.DefaultCellStyle.Format = "c";
            tbc2.DefaultCellStyle.Alignment = DataGridViewContentAlignment.MiddleRight;
            tbc2.DefaultCellStyle.Font = new Font(dataGridView1.Font, FontStyle.Bold);
            tbc2.DisplayIndex = 3;
            dgv.Columns.Add(tbc2);
        }

    }
}