using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Collections;

namespace MARS_Demo
{
    using System.Data.SqlClient;

    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

//     Das Beispiel demonstriert wie eine einzelne Bestellung aus Orders von der Northwind-Datenbank gelesen  
//     und benutzt wird, um den UnitsOnOrder-Bestand des verkauften Artikels entsprechend der verkauften Anzahl zu erhhen. 
//     Die typische Lsung (ohne MARS) erfordert zwei aufeinanderfolgende Verbindungen zur Datenbank, 
//     eine zum Lesen der Anzahl des verkauften Artikels und eine zum Aktualisieren der bestellten Einheiten.
//     Auerdem mssen die gelesenen Daten zwischengespeichert werden. 
//     Die Lsung mit MARS erfordert nur eine Verbindung und kommt ohne Datencache aus.

        // Es wird die Datenbankdatei Northwind.mdf verwendet, die sich im Anwendungsverzeichnis befindet:
        private const string CONNSTR = "Data Source=.\\SQLEXPRESS;AttachDbFilename=|DataDirectory|\\Northwind.mdf;Integrated Security=True;" +
                                 "User Instance=True";
        
        private const string OID = "10250";    // die OrderID (10248 ... 11069)

        // Beide Varianten benutzen hintereinander die folgenden SQL-Abfragen:
        private const string SQL1 = "SElECT * FROM [Order Details] WHERE OrderID = " + OID;
        private const string SQL2 = "UPDATE Products SET UnitsOnOrder=UnitsOnOrder + @anz WHERE (ProductID=@pid)";
               

        // Ohne MARS:
        private void button1_Click(object sender, EventArgs e)
        {
            // Der erste DataReader liest ProductID und Quantity aus der Order Details-Tabelle in den Datencache:

            ArrayList aIDs = new ArrayList();           // zum Zwischenspeichern der ProductID-Werte
            ArrayList aQts = new ArrayList();           // ...                   der Quantity-Werte
                        
            SqlConnection conn1 = new SqlConnection(CONNSTR);
            conn1.Open();
            SqlCommand cmd1 =  new SqlCommand(SQL1, conn1);
                       
            using (SqlDataReader reader = cmd1.ExecuteReader())
            {
                while (reader.Read())
                {
                    aIDs.Add(reader["ProductID"]);
                    aQts.Add(reader["Quantity"]);
                }
            }
            conn1.Close();

            // Der zweite DataReader aktualisiert die Products-Tabelle mit den Werten des Datencache:

            SqlConnection conn2 = new SqlConnection(CONNSTR);
            conn2.Open();
            SqlCommand cmd2 = new SqlCommand(SQL2, conn2);
            cmd2.Parameters.Add("@anz", SqlDbType.SmallInt);
            cmd2.Parameters.Add("@pid", SqlDbType.Int);
            for (int i = 0; i < aIDs.Count; i++)
            {
                cmd2.Parameters["@anz"].Value = aQts[i];
                cmd2.Parameters["@pid"].Value = aIDs[i];
                cmd2.ExecuteNonQuery();
            }
            conn2.Close();
            showResults();
        }

        // Mit MARS:
        private void button2_Click(object sender, EventArgs e)
        {
            SqlConnection conn = new SqlConnection(CONNSTR + ";MultipleActiveResultSets=True");   // !
            conn.Open();
            SqlCommand cmd1 = new SqlCommand(SQL1, conn);
            SqlCommand cmd2 = new SqlCommand(SQL2, conn);

            cmd2.Parameters.Add("@anz", SqlDbType.SmallInt);
            cmd2.Parameters.Add("@pid", SqlDbType.Int);
            using (SqlDataReader reader = cmd1.ExecuteReader())
            {
                while (reader.Read())
                {
                    cmd2.Parameters["@anz"].Value = reader["Quantity"];
                    cmd2.Parameters["@pid"].Value = reader["ProductID"];
                    cmd2.ExecuteNonQuery();
                }
            }
            conn.Close();
            showResults();
        }

        // Kontrollanzeige:
        private void showResults()
        {
            const string SQL = "SELECT ProductName, Quantity, UnitsOnOrder FROM Products, [Order Details] " +
                               "WHERE ([Order Details].OrderID = " + OID + ") AND (Products.ProductID = [Order Details].ProductID)";
            SqlDataAdapter da = new SqlDataAdapter(SQL, new SqlConnection(CONNSTR));
            DataTable dt = new DataTable();
            da.Fill(dt);
            dataGridView1.DataSource = dt;
        }
    
    }
}